@@ -1,8 +1,8 @@
-use itertools::Itertools;
-
+use rinex::sv;
 use rinex::epoch;
 use rinex::types::Type;
 use rinex::constellation::Constellation;
+use itertools::Itertools;
 
 fn main() {
     println!("**************************");
@@ -10,9 +10,7 @@ fn main() {
     println!("**************************");
 
     // example file
-    let navigation_file = std::path::PathBuf::from(
-        env!("CARGO_MANIFEST_DIR").to_owned() + "/data/NAV/V3/CBW100NLD_R_20210010000_01D_MN.rnx");
-    // parse example file
+    let navigation_file = env!("CARGO_MANIFEST_DIR").to_owned() + "/data/NAV/V3/CBW100NLD_R_20210010000_01D_MN.rnx";
     let rinex = rinex::Rinex::from_file(&navigation_file).unwrap();
 
     // header information
@@ -25,10 +23,9 @@ fn main() {
     assert_eq!(rinex.header.leap.is_some(), true);
 
     // (NAV) record analysis
-    if let Some(ref record) = rinex.record {
-        let record = record
-            .as_nav()
-            .unwrap();
+    let record = rinex.record
+        .as_nav()
+        .unwrap();
     //////////////////////////////
     // basic record browsing
     //////////////////////////////
@@ -62,7 +59,7 @@ fn main() {
     println!("\n------------- Matching epoch \"{:?}\" ----------\n{:#?}", to_match, matched); 
     
     // ----> zoom in on `E01` vehicule for that particular `epoch` 
-    let to_match = rinex::record::Sv::new(Constellation::Galileo, 1);
+    let to_match = sv::Sv::new(Constellation::Galileo, 1);
     let matched = &matched[&to_match];
     println!("\n------------- Adding Sv filter \"{:?}\" to previous epoch filter ----------\n{:#?}", to_match, matched); 
     // ----> zoom in on `E01` clock drift for that `epoch`
@@ -74,8 +71,7 @@ fn main() {
     // iterators + filters allow complex
     // pattern matching, data filtering and extraction
     ///////////////////////////////////////////////////
-    let record = rinex.record.unwrap();
-    let record = record
+    let record = rinex.record
         .as_nav()
         .unwrap();
 
@@ -83,12 +79,11 @@ fn main() {
     let epochs: Vec<_> = record
         .keys()
         .map(|k| k.date)
-        .sorted()
         .collect();
     println!("\n------------- Epochs ----------\n{:#?}", epochs); 
     
     // extract all data for `R24` vehicule 
-    let to_match = rinex::record::Sv::new(Constellation::Glonass, 24);
+    let to_match = sv::Sv::new(Constellation::Glonass, 24);
     let matched : Vec<_> = record
         .iter()
         .map(|(_epoch, sv)| { // dont care about epoch, sv filter
@@ -131,5 +126,4 @@ fn main() {
         .collect();
     println!("\n------------- Constellation: \"{:?}\" ----------\n{:#?}", to_match, matched); 
     }*/
-    }
 }
