@@ -1,6 +1,5 @@
-//! hatanaka.rs   
-//! structures and macros suites for
-//! RINEX OBS files compression and decompression.   
+//! Structures and macros for
+//! RINEX OBS file compression and decompression.   
 use crate::sv;
 use crate::header;
 use crate::is_comment;
@@ -23,12 +22,10 @@ pub enum KernelError {
 /// we can compress / decompress
 #[derive(Clone, Debug)]
 pub enum Dtype {
-    /// Numerical is intended to be used    
+    /// Numerical is intended to be used
     /// for observation data and clock offsets
     Numerical(i64),
-    /// Text represents epoch descriptors,   
-    /// GNSS descriptors,    
-    /// observation flags..
+    /// Text represents epoch descriptors, GNSS descriptors, observation flags..
     Text(String),
 }
 
@@ -51,11 +48,6 @@ impl Dtype {
     }
 }
 
-/// Fills given i64 vector with '0'
-fn zeros (array : &mut Vec<i64>) {
-    for i in 0..array.len() { array[i] = 0 }
-}
-
 /// `Kernel` is a structure to compress    
 /// or recover data using recursive defferential     
 /// equations as defined by Y. Hatanaka.   
@@ -101,8 +93,8 @@ impl Kernel {
         }
         // reset
         self.n = 0;
-        zeros(&mut self.state);
-        zeros(&mut self.p_state);
+        self.state.iter_mut().map(|x| *x = 0).count();
+        self.p_state.iter_mut().map(|x| *x = 0).count();
         // init
         self.init = data.clone();
         self.p_state[0] = data.as_numerical().unwrap_or(0);
@@ -135,7 +127,7 @@ impl Kernel {
     fn numerical_data_recovery (&mut self, data: i64) -> Dtype {
         self.n += 1;
         self.n = std::cmp::min(self.n, self.order);
-        zeros(&mut self.state);
+        self.state.iter_mut().map(|x| *x = 0).count();
         self.state[self.n] = data;
         for index in (0..self.n).rev() {
             self.state[index] = 
@@ -335,11 +327,12 @@ impl Decompressor {
                         Some(num)
                     },
                 };
-                // we can now fully recover the descriptor line
+                // we can now fully recover the epoch description 
                 let recovered_epoch =  // trick to recover textdiff
                     self.epo_krn.recover(Dtype::Text(String::from(" ")))?
                     .as_text()
                     .unwrap();
+                let recovered_epoch = recovered_epoch.as_str().trim_end();
                 match rnx_version.major {
                     1|2 => { // old RINEX
                         // system # id is appended
@@ -347,25 +340,21 @@ impl Decompressor {
                         let (epoch, systems) = recovered_epoch.split_at(32);
                         result.push_str(epoch);
                         let mut begin = 0;
-                        let n = num_integer::div_ceil(systems.len(), 12*3); // max sv per line
                         // terminate first line with required content
                         let end = std::cmp::min(begin+12*3, systems.len());
                         result.push_str(&systems[begin..end]);
                         // squeeze clock offset here, if any
                         if let Some(offset) = clock_offset {
-                            result.push_str(&format!("         {:3.12}", (offset as f64)/1000.0_f64))
+                            result.push_str(&format!("  {:3.9}", (offset as f64)/1000.0_f64))
                         }
-                        result.push_str("\n");
-                        begin += 12*3; // `systems` pointer
-                        for i in 1..n { // nb of missing lines, to fit remaining systems 
-                            let end = std::cmp::min(begin+12*3, systems.len());
-                            result.push_str("                                ");
-                            result.push_str(&systems[begin..end]);
-                            if i < n-1 {
-                                result.push_str("\n"); // otherwise,
-                                                // last line is terminated at the end of this block
+                        loop { // missing lines to fit remaining systems 
+                            begin += 12*3; // `systems` pointer
+                            if begin >= systems.len() {
+                                break
                             }
-                            begin += 12*3
+                            let end = std::cmp::min(begin+12*3, systems.len());
+                            result.push_str("\n                                ");
+                            result.push_str(&systems[begin..end]);
                         }
                     },
                     _ => { // modern RINEX
