@@ -23,7 +23,7 @@ use std::io::Write;
 use thiserror::Error;
 use std::str::FromStr;
 use itertools::Itertools;
-use std::collections::HashMap;
+use std::collections::{BTreeMap, HashMap};
 
 #[macro_export]
 /// Returns `true` if given `Rinex` line is a comment
@@ -96,42 +96,22 @@ pub enum Error {
 }
 
 #[derive(Error, Debug)]
-/// `RINEX` merge ops related errors
+/// `Merge` ops related errors
 pub enum MergeError {
     #[error("file types mismatch: cannot merge different `rinex`")]
     FileTypeMismatch,
-    #[error("not a \"standard\" File Merge Opts descriptor")]
-    MergeOptsDescriptionMismatch,
-    #[error("failed to parse File Merge boundary from Opts")]
+    #[error("failed to parse date field")] 
     ParseDateError(#[from] chrono::format::ParseError),
 }
 
-#[derive(Clone, Debug)]
-/// `RINEX` merging options
-pub struct MergeOpts {
-    /// optionnal program name
-    program: String,
-    /// timestamp where new file was appended
-    date: chrono::NaiveDateTime, 
+#[derive(Error, Debug)]
+/// `Split` ops related errors
+pub enum SplitError {
+    #[error("desired epoch is too early")]
+    EpochTooEarly,
+    #[error("desired epoch is too late")]
+    EpochTooLate,
 }
-
-/*impl std::str::FromStr for MergeOpts {
-    Err = MergeError;
-    /// Builds MergeOpts structure from "standard" RINEX comment line
-    fn from_str (line: &str) -> Result<Self, Self::Err> {
-        let (program, rem) = line.split_at(20);
-        let (ops, rem) = rem.split_at(20);
-        let (date, _) = rem.split_at(20);
-        if !opts.trim().eq("FILE MERGE") {
-            return Err(MergeError::MergeOptsDescriptionMismatch)
-        }
-        MergeOpts {
-            program: program.trim().to_string(),
-            date : chrono::DateTime::parse_from_str(date.split_at(16).0, "%Y%m%d %h%m%s")?, 
-        }
-    }
-}*/
-
 impl Rinex {
     /// Builds a new `RINEX` struct from given header & body sections
     pub fn new (header: header::Header, record: record::Record) -> Rinex {
@@ -197,7 +177,7 @@ impl Rinex {
     /// + either directly from optionnal information contained in `header`   
     /// + or (if not provided by header), by computing the average time interval between two successive epochs,    
     ///   in the extracted `record`. Only valid epochs (EpochFlag::Ok) contribute to the calculation in this case.   
-    ///   Returns None, in this case if record contains a unique epoch and calculation is not feasible.
+    ///   In this case, we return _None_ if calculation was not feasible (empty or single epoch)
     pub fn sampling_interval (&self) -> Option<std::time::Duration> {
         if let Some(interval) = self.header.sampling_interval {
             Some(std::time::Duration::from_secs(interval as u64))
@@ -304,19 +284,20 @@ impl Rinex {
     /// and returning closest comment (inside record) in time.    
     /// Usually, comments are associated to epoch events (anomalies) to describe what happened.   
     /// This method tries to locate a list of comments that were associated to the given timestamp 
-    pub fn event_description (&self, event: epoch::Epoch) -> Option<&str> {
+    pub fn event_description (&self, event: epoch::Epoch) -> Option<String> {
         let comments : Vec<_> = self.comments
             .iter()
-            .filter(|(k,v)| *k == &event)
-            .map(|(k,v)| v)
+            .filter(|(k,_)| *k == &event)
+            .map(|(_,v)| v)
             .flatten()
             .collect();
         if comments.len() > 0 {
-            Some(comments[0]) // TODO grab all content! by serializing into a single string
+            let comments = itertools::join(&comments, ", ");
+            Some(comments)
         } else {
             None
         }
-    } 
+    }
 
     /// Returns `true` if self is a `merged` RINEX file,   
     /// that means results from two or more separate RINEX files merged toghether.   
@@ -332,38 +313,274 @@ impl Rinex {
         false
     }
 
-    /// Returns list of epochs where RINEX merging operation(s) occurred.    
-    /// Epochs are determined either by the pseudo standard `FILE MERGE` comment description,
-    /// or by comment epochs inside the record
+    /// Returns list of epochs where RINEX merging operation(s) occurred.
+    /// Epochs are determined by the pseudo standard `FILE MERGE` comment description in header section.
     pub fn merge_boundaries (&self) -> Vec<chrono::NaiveDateTime> {
+        self.header.comments
+            .iter()
+            .flat_map(|s| {
+                if s.contains("FILE MERGE") {
+                    let content = s.split_at(40).1.trim();
+                    if let Ok(date) = chrono::NaiveDateTime::parse_from_str(content, "%Y%m%d %h%m%s") {
+                        Some(date)
+                    } else {
+                        None
+                    }
+                } else {
+                    None
+                }
+            })
+            .collect()
+    }
+
+    /// Splits merged `records` into seperate `records`.
+    /// Returns empty list if self is not a `Merged` file
+    pub fn split_records (&self) -> Vec<record::Record> {
+        let boundaries = self.merge_boundaries();
+        let mut result : Vec<record::Record> = Vec::with_capacity(boundaries.len());
         let epochs : Vec<&epoch::Epoch> = match self.header.rinex_type {
             types::Type::ObservationData => self.record.as_obs().unwrap().keys().collect(),
             types::Type::NavigationMessage => self.record.as_nav().unwrap().keys().collect(),
             types::Type::MeteoData => self.record.as_meteo().unwrap().keys().collect(),
         };
-        self.comments
-            .iter()
-            .filter(|(k,v)| {
-                v[0].contains("FILE MERGE") // TODO: wrapp all lines into a single one for testing purposes
-            })
-            .map(|(k,v)| k.date) // retain only timestamps
-            .collect()
-    }    
-
-    /// Splits merged RINEX `records` into list of records 
-    pub fn split (&self) -> Vec<record::Record> {
-        let mut result : Vec<record::Record> = Vec::with_capacity(2);
+        let mut t_0 = epochs[0].date;
+        for boundary in boundaries {
+            let included : Vec<_> = epochs
+                .iter()
+                .filter(|e| e.date >= t_0 && e.date < boundary)
+                .collect();
+            let rec = match self.header.rinex_type {
+                types::Type::NavigationMessage => {
+                    let rec : BTreeMap<_, _> = self.record.as_nav().unwrap()
+                        .iter()
+                        .filter(|(k, _)| k.date >= t_0 && k.date < boundary)
+                        .map(|(k, v)| (k.clone(),v.clone()))
+                        .collect();
+                    record::Record::NavRecord(rec)
+                },
+                types::Type::ObservationData => {
+                    let rec : BTreeMap<_, _> = self.record.as_obs().unwrap()
+                        .iter()
+                        .filter(|(k, _)| k.date >= t_0 && k.date < boundary)
+                        .map(|(k, v)| (k.clone(),v.clone()))
+                        .collect();
+                    record::Record::ObsRecord(rec)
+                },
+                types::Type::MeteoData => {
+                    let rec : BTreeMap<_, _> = self.record.as_meteo().unwrap()
+                        .iter()
+                        .filter(|(k, _)| k.date >= t_0 && k.date < boundary)
+                        .map(|(k, v)| (k.clone(),v.clone()))
+                        .collect();
+                    record::Record::MeteoRecord(rec)
+                },
+            };
+            result.push(rec);
+            t_0 = boundary 
+        }
         result
     }
 
-    /// Merges given RINEX into self, in teqc similar fashion.   
-    /// Header section are combined.    
-    /// Records are appended together at desired epoch boundary
-    pub fn merge (&mut self, rinex: &Self) -> Result<(), MergeError> {
-        if self.header.rinex_type != rinex.header.rinex_type {
+    /// Splits self into two seperate records, at desired `epoch`.
+    /// Self does not have to be a `Merged` file
+    pub fn split_records_at_epoch (&self, epoch: epoch::Epoch) -> Result<(record::Record,record::Record), SplitError> {
+        let epochs : Vec<&epoch::Epoch> = match self.header.rinex_type {
+            types::Type::ObservationData => self.record.as_obs().unwrap().keys().collect(),
+            types::Type::NavigationMessage => self.record.as_nav().unwrap().keys().collect(),
+            types::Type::MeteoData => self.record.as_meteo().unwrap().keys().collect(),
+        };
+        if epoch.date < epochs[0].date {
+            return Err(SplitError::EpochTooEarly)
+        }
+        if epoch.date > epochs[0].date {
+            return Err(SplitError::EpochTooLate)
+        }
+        let rec0 : record::Record = match self.header.rinex_type {
+            types::Type::NavigationMessage => {
+                let rec = self.record.as_nav()
+                    .unwrap()
+                        .iter()
+                        .flat_map(|(k, v)| {
+                            if k.date < epoch.date {
+                                Some((k, v))
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|(k,v)| (k.clone(),v.clone())) // BTmap collect() derefencing 
+                        .collect();
+                record::Record::NavRecord(rec)
+            },
+            types::Type::ObservationData => {
+                let rec = self.record.as_obs()
+                    .unwrap()
+                        .iter()
+                        .flat_map(|(k, v)| {
+                            if k.date < epoch.date {
+                                Some((k, v))
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|(k,v)| (k.clone(),v.clone())) // BTmap collect() derefencing 
+                        .collect();
+                record::Record::ObsRecord(rec)
+            },
+            types::Type::MeteoData => {
+                let rec = self.record.as_meteo()
+                    .unwrap()
+                        .iter()
+                        .flat_map(|(k, v)| {
+                            if k.date < epoch.date {
+                                Some((k, v))
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|(k,v)| (k.clone(),v.clone())) // BTmap collect() derefencing 
+                        .collect();
+                record::Record::MeteoRecord(rec)
+            },
+        };
+        let rec1 : record::Record = match self.header.rinex_type {
+            types::Type::NavigationMessage => {
+                let rec = self.record.as_nav()
+                    .unwrap()
+                        .iter()
+                        .flat_map(|(k, v)| {
+                            if k.date >= epoch.date {
+                                Some((k, v))
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|(k,v)| (k.clone(),v.clone())) // BTmap collect() derefencing 
+                        .collect();
+                record::Record::NavRecord(rec)
+            },
+            types::Type::ObservationData => {
+                let rec = self.record.as_obs()
+                    .unwrap()
+                        .iter()
+                        .flat_map(|(k, v)| {
+                            if k.date >= epoch.date {
+                                Some((k, v))
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|(k,v)| (k.clone(),v.clone())) // BTmap collect() derefencing 
+                        .collect();
+                record::Record::ObsRecord(rec)
+            },
+            types::Type::MeteoData => {
+                let rec = self.record.as_meteo()
+                    .unwrap()
+                        .iter()
+                        .flat_map(|(k, v)| {
+                            if k.date >= epoch.date {
+                                Some((k, v))
+                            } else {
+                                None
+                            }
+                        })
+                        .map(|(k,v)| (k.clone(),v.clone())) // BTmap collect() derefencing 
+                        .collect();
+                record::Record::MeteoRecord(rec)
+            },
+        };
+        Ok((rec0,rec1))
+    }
+
+    /// Splits Self into separate `RINEX` structures. 
+    /// Self must be a `Merged` file.
+    /// Header sections are simply copied.
+    /// Records are built using the .split_records() method.
+    pub fn split (&self) -> Vec<Self> {
+        let records = self.split_records();
+        let mut result : Vec<Self> = Vec::with_capacity(records.len());
+        for rec in records {
+            result.push(Self {
+                header: self.header.clone(),
+                comments: self.comments.clone(),
+                record: rec.clone(),
+            })
+        }
+        result
+    }
+
+    /// Merges given RINEX into self, in similar fashion as `teqc` program.  
+    /// Header section attributes are combined:    
+    ///   + self.attributes are preserved / preferred
+    ///   + if an attribute is not known to self, it gets created
+    /// Record sections are merged together, respecting epochs chronology. 
+    pub fn merge (&mut self, other: &Self) -> Result<(), MergeError> {
+        if self.header.rinex_type != other.header.rinex_type {
             return Err(MergeError::FileTypeMismatch)
         }
-        Ok(())
+        // grab Self:: + Other:: `epochs`
+        let (epochs, other_epochs) : (Vec<&epoch::Epoch>, Vec<&epoch::Epoch>) = match self.header.rinex_type {
+            types::Type::ObservationData => {
+                (self.record.as_obs().unwrap().keys().collect(),
+                other.record.as_obs().unwrap().keys().collect())
+            },
+            types::Type::NavigationMessage => {
+                (self.record.as_nav().unwrap().keys().collect(),
+                other.record.as_nav().unwrap().keys().collect())
+            },
+            types::Type::MeteoData => {
+                (self.record.as_meteo().unwrap().keys().collect(),
+                other.record.as_meteo().unwrap().keys().collect())
+            },
+        };
+        if epochs.len() == 0 { // self is empty
+            self.record = other.record.clone();
+            Ok(()) // --> self is overwritten
+        } else if other_epochs.len() == 0 { // nothing to merge
+            Ok(()) // --> self is untouched
+        } else {
+            // add Merge op descriptor
+            let now = chrono::offset::Utc::now();
+            self.header.comments.push(format!(
+                "rustrnx-{:<20} FILE MERGE          {} UTC", 
+                env!("CARGO_PKG_VERSION"),
+                now.format("%Y%m%d %H%M%S")));
+            // determine min epoch to begin with 
+            let min = (epochs.iter().min(), other_epochs.iter().min());
+            let other_rec : BTreeMap<_,_> = match self.header.rinex_type {
+                types::Type::NavigationMessage => other.record.as_nav().unwrap().iter().collect(),
+                _ => panic!("salut"),
+            };
+            match self.header.rinex_type {
+                /*types::Type::NavigationMessage => {
+                    for (entry, value) in other_rec {
+                        self.record.as_nav().unwrap().insert(*entry, value.clone());
+                    }
+                },*/
+                _ => panic!("salut"),
+            }
+            Ok(())
+        }
+    }
+
+    /// Decimates `record` data with a minimal sampling interval to match
+    /// (modifies Self in place)
+    pub fn decimate (&mut self, interval: std::time::Duration) {
+        /*match self.header.rinex_type {
+            types::Type::NavigationMessage => {
+                let mut record = self.record.as_nav().unwrap();
+                let mut iter = record.iter_mut(); 
+                for i in 0..iter.len() { 
+                }
+            },
+            _ => panic!("youhou"),
+        };*/
+    }
+
+    /// Decimates `record` data with a minimal sampling interval to match
+    /// and returns resulting `record` 
+    pub fn decimate_copy (&self, interval: std::time::Duration) -> record::Record {
+        self.record.clone()
     }
 
     /// Writes self into given file.   
