@@ -1,9 +1,8 @@
-//! record.rs describes `RINEX` file content
+//! `RINEX` file content description and parsing
 use thiserror::Error;
 use std::fs::File;
 use std::str::FromStr;
-use std::io::{self, prelude::*, BufReader};
-use itertools::Itertools;
+use std::io::{prelude::*, BufReader};
 use std::collections::{BTreeMap, HashMap};
 
 use crate::sv;
